from __future__ import unicode_literals

from django import forms
from admin_page_lock.mixins.base_mixin import BaseLockingMixin
from admin_page_lock.settings import DISABLE


class PageLockAdminMixin(BaseLockingMixin):
    def change_view(self, req, object_id, form_url='', extra_context=None):
        if not DISABLE and self.lock_change_view:
            result = self._open_page_connection_data(req)
            extra_context = {} if extra_context is None else extra_context
            extra_context.update(self._add_extra_content(req, result))

        return super(PageLockAdminMixin, self).change_view(
            req,
            object_id,
            form_url,
            extra_context=extra_context
        )

    def changelist_view(self, req, extra_context=None):
        if not DISABLE and self.lock_changelist_view:
            result = self._open_page_connection_data(req)
            extra_context = {} if extra_context is None else extra_context
            extra_context.update(self._add_extra_content(req, result))

        return super(PageLockAdminMixin, self).changelist_view(
            req,
            extra_context=extra_context
        )

    def get_actions(self, req):
        # TODO(vstefka) hide only actions that can change db.
        if not self._is_locked(req):
            return super(PageLockAdminMixin, self).get_actions(req)

        return None

    def get_form(self, req, obj=None, **kwargs):
        form = super(PageLockAdminMixin, self).get_form(req, obj, **kwargs)

        if self._is_locked(req):
            for field_name in [
                i
                for i, j in form.base_fields.items()
                if issubclass(type(j), forms.FileField)
            ]:
                form.base_fields[field_name].widget.attrs['disabled'] = 'disabled'  # noqa: E501

        return form

    def get_prepopulated_fields(self, req, obj=None):
        # It is not possible to pre-populate fields with readonly fields.
        if self._is_locked(req):
            return {}

        return super(PageLockAdminMixin, self).get_prepopulated_fields(req, obj)

    def get_readonly_fields(self, req, obj=None):
        readonly_fields = super(PageLockAdminMixin, self).get_readonly_fields(req, obj)  # noqa: E501

        if self._is_locked(req):
            readonly_fields = list(readonly_fields) + list(
                [field.name for field in self.opts.local_fields] +
                [field.name for field in self.opts.local_many_to_many]
            )

        return tuple(readonly_fields)

    def has_add_permission(self, req):
        can_add = super(PageLockAdminMixin, self).has_add_permission(req)

        if can_add and not self._is_locked(req):
            return True

        return False

    def has_delete_permission(self, req, obj=None):
        can_delete = super(PageLockAdminMixin, self).has_delete_permission(req, obj)  # noqa: E501

        if can_delete and not self._is_locked(req):
            return True

        return False

    @property
    def media(self):
        media = super(PageLockAdminMixin, self).media + forms.Media(
            js=(
                'js/page_lock.js',
            ),
        )

        return media



from __future__ import unicode_literals

import importlib

from django.utils.crypto import get_random_string
from admin_page_lock.settings import (
    HANDLER_CLASS,
    MODEL
)


def get_page_lock_class(class_path):
    module_name, class_name = class_path.rsplit('.', 1)
    try:
        module = importlib.import_module(module_name)
        page_lock_class = getattr(module, class_name)
    except (ImportError, AttributeError):
        raise

    return page_lock_class


def get_page_lock_classes():
    handler_class = get_page_lock_class(HANDLER_CLASS)
    model_class = get_page_lock_class(MODEL)

    return handler_class, model_class


def get_new_csrf_token(csfr_key_lenght):
    return get_random_string(csfr_key_lenght)



from __future__ import unicode_literals

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

# API interval.
API_INTERVAL_DEFAULT = 30000  # [ms]
API_INTERVAL_REFERENCE = 'PAGE_LOCK_API_INTERVAL'
API_INTERVAL = getattr(
    settings, API_INTERVAL_REFERENCE, API_INTERVAL_DEFAULT)

# User can open same page in more then one tabs.
CAN_OPEN_MORE_TABS_DEFAULT = False
CAN_OPEN_MORE_TABS_REFERENCE = 'PAGE_LOCK_CAN_OPEN_MORE_TABS'
CAN_OPEN_MORE_TABS = getattr(
    settings, CAN_OPEN_MORE_TABS_REFERENCE, CAN_OPEN_MORE_TABS_DEFAULT)

# CRFS token.
DISABLE_CRSF_TOKEN_DEFAULT = False
DISABLE_CRSF_TOKEN_REFERENCE = 'PAGE_LOCK_DISABLE_CRSF_TOKEN'
DISABLE_CRSF_TOKEN = getattr(
    settings, DISABLE_CRSF_TOKEN_REFERENCE, DISABLE_CRSF_TOKEN_DEFAULT)

# Disable locking.
DISABLE_DEFAULT = False
DISABLE_REFERENCE = 'PAGE_LOCK_DISABLE'
DISABLE = getattr(settings, DISABLE_REFERENCE, DISABLE_DEFAULT)

# Handler.
HANDLER_CLASS_DEFAULT = 'admin_page_lock.handlers.PageLockHandler'
HANDLER_CLASS_REFERENCE = 'PAGE_LOCK_HANDLER_CLASS'
HANDLER_CLASS = getattr(
    settings, HANDLER_CLASS_REFERENCE, HANDLER_CLASS_DEFAULT)

# Handler functions:
HANDLER_FUNCTION_CLOSE_PAGE_CONNECTION = 'close_page_connection'
HANDLER_FUNCTION_GET_PAGE_INFO = 'get_page_info'
HANDLER_FUNCTION_OPEN_PAGE_CONNECTION = 'open_page_connection'

# Home page redirect.
HOMEPAGE_DEFAULT = '/'
HOMEPAGE_REFERENCE = 'PAGE_LOCK_HOMEPAGE'
HOMEPAGE = getattr(settings, HOMEPAGE_REFERENCE, HOMEPAGE_DEFAULT)

# Keep DB locks (possible to see history).
KEEP_DB_LOCKS_DEFAULT = False
KEEP_DB_LOCKS_REFERENCE = 'PAGE_LOCK_KEEP_DB_LOCKS'
KEEP_DB_LOCKS = getattr(settings, KEEP_DB_LOCKS_REFERENCE, KEEP_DB_LOCKS_DEFAULT)  # noqa: E501

# Messages (see documentation).
MESSAGES_DEFAUL = {
    'message_locked': _('Page is locked.'),
    'message_reload': _('You are the one who can edit. Click "RELOAD" button to stay on the page.'),  # noqa: E501
    'message_refresh': _('Click "REFRESH" button to start editting.'),
    'message_problem': _('Something happened, you will be redirected to homepage.'),  # noqa: E501
}
MESSAGES_REFERENCE = 'PAGE_LOCK_MESSAGES'
MESSAGES = getattr(settings, MESSAGES_REFERENCE, MESSAGES_DEFAUL)

# Times (see documentation).
TIMEOUT_DEFAULT = 6000  # [s]
TIMEOUT_REFERENCE = 'PAGE_LOCK_TIMEOUT'
TIMEOUT = getattr(settings, TIMEOUT_REFERENCE, TIMEOUT_DEFAULT)

# Model.
MODEL_DEFAULT = 'admin_page_lock.models.database_model.DatabasePageLockModel'
MODEL_REFERENCE = 'PAGE_LOCK_MODEL'
MODEL = getattr(settings, MODEL_REFERENCE, MODEL_DEFAULT)

# Redis.
REDIS_PREFIX = 'lock-page'
REDIS_SETTINGS_DEFAULT = {
    'host': '127.0.0.1',
    'port': 6379,
    'password': '',
    'timeout': 5
}
REDIS_SETTINGS_REFERENCE = 'PAGE_LOCK_REDIS_SETTINGS'
REDIS_SETTINGS = getattr(
    settings, REDIS_SETTINGS_REFERENCE, REDIS_SETTINGS_DEFAULT)

# Url.
URL_IGNORE_PARAMETERS_DEFAULT = True
URL_IGNORE_PARAMETERS_REFERENCE = 'PAGE_LOCK_URL_IGNORE_PARAMETERS'
URL_IGNORE_PARAMETERS = getattr(
    settings, URL_IGNORE_PARAMETERS_REFERENCE, URL_IGNORE_PARAMETERS_DEFAULT)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import admin_page_lock.models.base_model


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DatabasePageLockModel',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('url', models.URLField()),
                ('url_parameters', models.CharField(max_length=1024, null=True, blank=True)),
                ('active', models.BooleanField(default=True)),
                ('user_reference', models.CharField(max_length=255, null=True, blank=True)),
                ('locked', models.BooleanField(default=True)),
                ('locked_at', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('locked_out', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ('locked_at',),
            },
            bases=(admin_page_lock.models.base_model.BasePageLockModel, models.Model),
        ),
    ]



from __future__ import unicode_literals

import json

from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _
from admin_page_lock.settings import (
    MESSAGES,
    URL_IGNORE_PARAMETERS
)
from urlparse import (
    parse_qsl,
    urlparse,
    urlsplit,
)


class BasePageLockModel(object):
    @classmethod
    def _get_page_full_url(cls, req):
        if hasattr(req, 'body') and 'url' in req.body:
            # Get `url` from body (JS call).
            page_full_url = json.loads(req.body)['url']
        else:
            # Get `url` by request function (Django view/adminview).
            page_full_url = req.get_full_path()

        return page_full_url

    @classmethod
    def _get_page_url_parameters(cls, req):
        # Returns string containing query parameters of the `url` with `_`
        # instead of `=`.
        if URL_IGNORE_PARAMETERS:
            return ''

        page_full_url = cls._get_page_full_url(req)

        try:
            parse_result = urlparse(page_full_url)
        except KeyError:
            raise

        if parse_result.query:
            return '_'.join(
                '_'.join([i, j])
                for i, j in parse_qsl(parse_result.query)
            )

        return ''

    @classmethod
    def _get_page_url(cls, req):
        # Returns url without parameters.
        page_full_url = cls._get_page_full_url(req)

        try:
            parse_result = urlsplit(page_full_url)
        except KeyError:
            raise

        return parse_result.path

    @classmethod
    def _get_session_key(cls, req):
        if hasattr(req, 'session'):
            return req.session.session_key

        return None

    @classmethod
    def _get_user_reference(cls, req):
        # Get username.
        username = cls._get_username(req)
        if username is not None:
            return str(username)

        # Get section.
        session_key = cls._get_session_key(req)
        if session_key is not None:
            return str(session_key)

        # Get user_reference from post.
        # curl -X POST -H "Accept: application/json"
        # -d '{"user_reference": "vstefka", "url": "page1"}'
        # "http://127.0.0.1:8000/page_lock/get_page_info/"
        try:
            return json.loads(req.body)['user_reference']
        except (KeyError, ValueError):
            pass

        raise Exception(_('User reference is not defined.'))

    @classmethod
    def _get_username(cls, req):
        if hasattr(req, 'user'):
            return req.user.username

        return None

    def deactivate(cls, page_settings):
        raise ImproperlyConfigured(
            _('Function: "close_page_connection" is not implemented'))

    @classmethod
    def get_data(cls, req, page_settings):
        raise ImproperlyConfigured(
            _('Function: "_get_data" is not implemented'))

    @classmethod
    def get_page_settings(cls, req):
        return {
            'messages': {
                i: str(j)
                for i, j in MESSAGES.items()
            },
            'page_url_parameters': cls._get_page_url_parameters(req),
            'page_url': cls._get_page_url(req),
            'req': req,
            'user_reference': cls._get_user_reference(req)
        }

    @classmethod
    def set_data(cls, req, page_settings, data):
        raise ImproperlyConfigured(
            _('Function: "_set_data" is not implemented'))



from __future__ import unicode_literals

import imp
import os

from setuptools import (
    find_packages,
    setup
)


ROOT = os.path.realpath(os.path.join(os.path.dirname(__file__)))
README_FILE = os.path.join(ROOT, 'README.md')
INIT = os.path.join(ROOT, 'admin_page_lock', '__init__.py')
APP = imp.load_source('page_lock', INIT)

# Allow setup.py to be run from any path.
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


def fread(fname):
    return open(fname).read()


setup(
    name=APP.NAME,
    version=APP.VERSION,
    packages=find_packages(),
    include_package_data=True,
    license='Apache License, Version 2.0',
    description='Page Lock application prevents users being able to edit '
                'same page defined by its unique URL at the same time. '
                'The application is tailored to django admin implementation.',
    long_description=fread(README_FILE),
    url='https://github.com/ShowMax/django-admin-page-lock',
    author='Vojtech Stefka',
    author_email='oss+djangopage@showmax.com',
    keywords=['django', 'admin', 'locking', 'concurrency'],
    install_requires=[],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    platforms=['any'],
)
